import { __decorate, __param } from 'vscode/external/tslib/tslib.es6.js';
import { $, clearNode, setVisibility, append, addDisposableListener, EventType, EventHelper } from 'vscode/vscode/vs/base/browser/dom';
import { createInstantHoverDelegate } from 'vscode/vscode/vs/base/browser/ui/hover/hoverDelegateFactory';
import { Emitter } from 'vscode/vscode/vs/base/common/event';
import { Disposable, DisposableStore } from 'vscode/vscode/vs/base/common/lifecycle';
import { basename, dirname } from 'vscode/vscode/vs/base/common/path';
import { URI } from 'vscode/vscode/vs/base/common/uri';
import { Range } from 'vscode/vscode/vs/editor/common/core/range';
import { localize } from 'vscode/vscode/vs/nls';
import { FileKind } from 'vscode/vscode/vs/platform/files/common/files';
import { IFileService } from 'vscode/vscode/vs/platform/files/common/files.service';
import { IHoverService } from 'vscode/vscode/vs/platform/hover/browser/hover.service';
import { IInstantiationService } from 'vscode/vscode/vs/platform/instantiation/common/instantiation';
import { IOpenerService } from 'vscode/vscode/vs/platform/opener/common/opener.service';
import { ResourceLabels } from 'vscode/vscode/vs/workbench/browser/labels';
import { ChatResponseReferencePartStatusKind } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatService';

let ChatAttachmentsContentPart = class ChatAttachmentsContentPart extends Disposable {
    constructor(variables, contentReferences = [], workingSet = [], domNode = $('.chat-attached-context'), instantiationService, openerService, hoverService, fileService) {
        super();
        this.variables = variables;
        this.contentReferences = contentReferences;
        this.workingSet = workingSet;
        this.domNode = domNode;
        this.instantiationService = instantiationService;
        this.openerService = openerService;
        this.hoverService = hoverService;
        this.fileService = fileService;
        this.attachedContextDisposables = this._register(( (new DisposableStore())));
        this._onDidChangeVisibility = this._register(( (new Emitter())));
        this._contextResourceLabels = this.instantiationService.createInstance(ResourceLabels, { onDidChangeVisibility: this._onDidChangeVisibility.event });
        this.initAttachedContext(domNode);
    }
    initAttachedContext(container) {
        clearNode(container);
        this.attachedContextDisposables.clear();
        setVisibility(Boolean(this.variables.length), this.domNode);
        const hoverDelegate = this.attachedContextDisposables.add(createInstantHoverDelegate());
        this.variables.forEach(async (attachment) => {
            const file = URI.isUri(attachment.value) ? attachment.value : attachment.value && typeof attachment.value === 'object' && 'uri' in attachment.value && URI.isUri(attachment.value.uri) ? attachment.value.uri : undefined;
            const range = attachment.value && typeof attachment.value === 'object' && 'range' in attachment.value && Range.isIRange(attachment.value.range) ? attachment.value.range : undefined;
            if (file && attachment.isFile && this.workingSet.find(entry => ( (entry.toString())) === ( (file.toString())))) {
                return;
            }
            const widget = append(container, $('.chat-attached-context-attachment.show-file-icons'));
            const label = this._contextResourceLabels.create(widget, { supportIcons: true, hoverDelegate, hoverTargetOverrride: widget });
            const correspondingContentReference = this.contentReferences.find((ref) => typeof ref.reference === 'object' && 'variableName' in ref.reference && ref.reference.variableName === attachment.name);
            const isAttachmentOmitted = correspondingContentReference?.options?.status?.kind === ChatResponseReferencePartStatusKind.Omitted;
            const isAttachmentPartialOrOmitted = isAttachmentOmitted || correspondingContentReference?.options?.status?.kind === ChatResponseReferencePartStatusKind.Partial;
            let ariaLabel;
            if (file && attachment.isFile) {
                const fileBasename = basename(file.path);
                const fileDirname = dirname(file.path);
                const friendlyName = `${fileBasename} ${fileDirname}`;
                if (isAttachmentOmitted) {
                    ariaLabel = range ? ( localize(
                        10832,
                        "Omitted: {0}, line {1} to line {2}.",
                        friendlyName,
                        range.startLineNumber,
                        range.endLineNumber
                    )) : ( localize(10833, "Omitted: {0}.", friendlyName));
                }
                else if (isAttachmentPartialOrOmitted) {
                    ariaLabel = range ? ( localize(
                        10834,
                        "Partially attached: {0}, line {1} to line {2}.",
                        friendlyName,
                        range.startLineNumber,
                        range.endLineNumber
                    )) : ( localize(10835, "Partially attached: {0}.", friendlyName));
                }
                else {
                    ariaLabel = range ? ( localize(
                        10836,
                        "Attached: {0}, line {1} to line {2}.",
                        friendlyName,
                        range.startLineNumber,
                        range.endLineNumber
                    )) : ( localize(10837, "Attached: {0}.", friendlyName));
                }
                label.setFile(file, {
                    fileKind: FileKind.FILE,
                    hidePath: true,
                    range,
                    title: correspondingContentReference?.options?.status?.description
                });
            }
            else if (attachment.isImage) {
                ariaLabel = ( localize(10838, "Attached image, {0}", attachment.name));
                const hoverElement = $('div.chat-attached-context-hover');
                hoverElement.setAttribute('aria-label', ariaLabel);
                const pillIcon = $('div.chat-attached-context-pill', {}, $('span.codicon.codicon-file-media'));
                const textLabel = $('span.chat-attached-context-custom-text', {}, attachment.name);
                widget.appendChild(pillIcon);
                widget.appendChild(textLabel);
                let buffer;
                try {
                    if (attachment.value instanceof URI) {
                        const readFile = await this.fileService.readFile(attachment.value);
                        buffer = readFile.value.buffer;
                    }
                    else {
                        buffer = attachment.value;
                    }
                    await this.createImageElements(buffer, widget, hoverElement);
                }
                catch (error) {
                    console.error('Error processing attachment:', error);
                }
                widget.style.position = 'relative';
                if (!this.attachedContextDisposables.isDisposed) {
                    this.attachedContextDisposables.add(this.hoverService.setupManagedHover(hoverDelegate, widget, hoverElement));
                }
            }
            else {
                const attachmentLabel = attachment.fullName ?? attachment.name;
                const withIcon = attachment.icon?.id ? `$(${attachment.icon.id}) ${attachmentLabel}` : attachmentLabel;
                label.setLabel(withIcon, correspondingContentReference?.options?.status?.description);
                ariaLabel = ( localize(10839, "Attached context: {0}.", attachment.name));
            }
            if (isAttachmentPartialOrOmitted) {
                widget.classList.add('warning');
            }
            const description = correspondingContentReference?.options?.status?.description;
            if (isAttachmentPartialOrOmitted) {
                ariaLabel = `${ariaLabel}${description ? ` ${description}` : ''}`;
                for (const selector of ['.monaco-icon-suffix-container', '.monaco-icon-name-container']) {
                    const element = label.element.querySelector(selector);
                    if (element) {
                        element.classList.add('warning');
                    }
                }
            }
            if (file) {
                widget.style.cursor = 'pointer';
                if (!this.attachedContextDisposables.isDisposed) {
                    this.attachedContextDisposables.add(addDisposableListener(widget, EventType.CLICK, async (e) => {
                        EventHelper.stop(e, true);
                        this.openerService.open(file, {
                            fromUserGesture: true,
                            editorOptions: {
                                selection: range
                            }
                        });
                    }));
                }
            }
            widget.ariaLabel = ariaLabel;
            widget.tabIndex = 0;
        });
    }
    async createImageElements(buffer, widget, hoverElement) {
        const blob = ( (new Blob([buffer], { type: 'image/png' })));
        const url = URL.createObjectURL(blob);
        const img = $('img.chat-attached-context-image', { src: url, alt: '' });
        const pillImg = $('img.chat-attached-context-pill-image', { src: url, alt: '' });
        const pill = $('div.chat-attached-context-pill', {}, pillImg);
        const existingPill = widget.querySelector('.chat-attached-context-pill');
        if (existingPill) {
            existingPill.replaceWith(pill);
        }
        hoverElement.appendChild(img);
    }
};
ChatAttachmentsContentPart = ( (__decorate([
    ( (__param(4, IInstantiationService))),
    ( (__param(5, IOpenerService))),
    ( (__param(6, IHoverService))),
    ( (__param(7, IFileService)))
], ChatAttachmentsContentPart)));

export { ChatAttachmentsContentPart };
